@@ -5,14 +5,15 @@ import json
 from pprint import pformat
 import sys
 import time
-import warnings
+from types import MethodType
 from typing import Any, Callable, Dict, List, Optional, Sequence, Set, Tuple, TypeVar, Union, get_type_hints
 from typing_inspect import is_literal_type, get_args, get_origin, is_union_type
 
 from tap.utils import (
     get_class_variables,
-    get_dest,
+    get_argument_name,
     get_git_root,
+    get_dest,
     get_git_url,
     has_git,
     has_uncommitted_changes,
@@ -101,14 +102,19 @@ class Tap(ArgumentParser):
         :param kwargs: Keyword arguments.
         """
         # Get variable name
-        variable = get_dest(*name_or_flags, **kwargs)
+        variable = get_argument_name(*name_or_flags)
 
         # Get default if not specified
         if hasattr(self, variable):
             kwargs['default'] = kwargs.get('default', getattr(self, variable))
 
         # Set required if option arg
-        if is_option_arg(*name_or_flags) and variable != 'help':
+        if (
+            is_option_arg(*name_or_flags)
+            and variable != 'help'
+            and 'default' not in kwargs
+            and kwargs.get('action') != 'version'
+        ):
             kwargs['required'] = kwargs.get('required', not hasattr(self, variable))
 
         # Set help if necessary
@@ -174,16 +180,18 @@ class Tap(ArgumentParser):
                 ):
                     var_type, kwargs['choices'] = get_literals(get_args(var_type)[0], variable)
                 elif var_type not in SUPPORTED_DEFAULT_TYPES:
+                    arg_params = "required=True" if kwargs["required"] else f"default={getattr(self, variable)}"
                     raise ValueError(
                         f'Variable "{variable}" has type "{var_type}" which is not supported by default.\n'
                         f'Please explicitly add the argument to the parser by writing:\n\n'
                         f'def add_arguments(self) -> None:\n'
-                        f'    self.add_argument("--{variable}", type=func, {"required=True" if kwargs["required"] else f"default={getattr(self, variable)}"})\n\n'
+                        f'    self.add_argument("--{variable}", type=func, {arg_params})\n\n'
                         f'where "func" maps from str to {var_type}.')
 
                 if var_type in SUPPORTED_DEFAULT_BOXED_TYPES:
                     # If List or Set type, set nargs
-                    if var_type in SUPPORTED_DEFAULT_COLLECTION_TYPES:
+                    if (var_type in SUPPORTED_DEFAULT_COLLECTION_TYPES
+                            and kwargs.get('action') not in {'append', 'append_const'}):
                         kwargs['nargs'] = kwargs.get('nargs', '*')
 
                     # Extract boxed type for Optional, List, Set
@@ -205,15 +213,19 @@ class Tap(ArgumentParser):
                         kwargs['type'] = boolean_type
                         kwargs['choices'] = [True, False]  # this makes the help message more helpful
                     else:
-                        kwargs['action'] = kwargs.get('action', f'store_{"true" if kwargs["required"] or not kwargs["default"] else "false"}')
-                else:
+                        action_cond = "true" if kwargs.get("required", False) or not kwargs["default"] else "false"
+                        kwargs['action'] = kwargs.get('action', f'store_{action_cond}')
+                elif kwargs.get('action') not in {'count', 'append_const'}:
                     kwargs['type'] = var_type
 
+        if self._underscores_to_dashes:
+            name_or_flags = [name_or_flag.replace('_', '-') for name_or_flag in name_or_flags]
+
         super(Tap, self).add_argument(*name_or_flags, **kwargs)
 
     def add_argument(self, *name_or_flags, **kwargs) -> None:
         """Adds an argument to the argument buffer, which will later be passed to _add_argument."""
-        variable = get_dest(*name_or_flags, **kwargs)
+        variable = get_argument_name(*name_or_flags)
         self.argument_buffer[variable] = (name_or_flags, kwargs)
 
     def _add_arguments(self) -> None:
@@ -224,8 +236,7 @@ class Tap(ArgumentParser):
                 name_or_flags, kwargs = self.argument_buffer[variable]
                 self._add_argument(*name_or_flags, **kwargs)
             else:
-                flag_name = variable.replace('_', '-') if self._underscores_to_dashes else variable
-                self._add_argument(f'--{flag_name}')
+                self._add_argument(f'--{variable}')
 
         # Add any arguments that were added manually in add_arguments but aren't class variables (in order)
         for variable, (name_or_flags, kwargs) in self.argument_buffer.items():
@@ -363,7 +374,11 @@ class Tap(ArgumentParser):
         class_dict = {
             var: val
             for var, val in class_dict.items()
-            if not (var.startswith('_') or callable(val) or isinstance(val, staticmethod))
+            if not (var.startswith('_')
+                    or callable(val)
+                    or isinstance(val, staticmethod)
+                    or isinstance(val, classmethod)
+                    or isinstance(val, property))
         }
 
         return class_dict
@@ -376,29 +391,42 @@ class Tap(ArgumentParser):
 
     def _get_class_variables(self) -> OrderedDict:
         """Returns an OrderedDict mapping class variables names to their additional information."""
+        class_variable_names = self._get_class_dict().keys() | self._get_annotations().keys()
+
         try:
             class_variables = self._get_from_self_and_super(
                 extract_func=lambda super_class: get_class_variables(super_class),
                 dict_type=OrderedDict
             )
+
+            # Handle edge-case of source code modification while code is running
+            variables_to_add = class_variable_names - class_variables.keys()
+            variables_to_remove = class_variables.keys() - class_variable_names
+
+            for variable in variables_to_add:
+                class_variables[variable] = {'comment': ''}
+
+            for variable in variables_to_remove:
+                class_variables.pop(variable)
         # Exception if inspect.getsource fails to extract the source code
         except Exception:
             class_variables = OrderedDict()
-            for variable in self._get_class_dict().keys():
+            for variable in class_variable_names:
                 class_variables[variable] = {'comment': ''}
 
         return class_variables
 
     def _get_argument_names(self) -> Set[str]:
         """Returns a list of variable names corresponding to the arguments."""
-        return (set(self._get_class_dict().keys()) |
-                set(self._annotations.keys()) |
-                set(self.argument_buffer.keys()) - {'help'}) 
+        return ({get_dest(*name_or_flags, **kwargs)
+                 for name_or_flags, kwargs in self.argument_buffer.values()} |
+                set(self._get_class_dict().keys()) |
+                set(self._annotations.keys())) - {'help'}
 
     def as_dict(self) -> Dict[str, Any]:
         """Returns the member variables corresponding to the parsed arguments.
 
-        Note: This does not include attributes set directly on an instance 
+        Note: This does not include attributes set directly on an instance
         (e.g. arg is not included in MyTap().arg = "hi")
 
         :return: A dictionary mapping each argument's name to its value.
@@ -406,7 +434,22 @@ class Tap(ArgumentParser):
         if not self._parsed:
             raise ValueError('You should call `parse_args` before retrieving arguments.')
 
-        return {var: getattr(self, var) for var in self._get_argument_names()}
+        self_dict = self.__dict__
+        class_dict = {key: val for key, val in type(self).__dict__.items() if key not in self_dict}
+        stored_dict = {**self_dict, **class_dict}
+
+        stored_dict = {
+            var: getattr(self, var)
+            for var, val in stored_dict.items()
+            if not (var.startswith('_')
+                    or isinstance(val, MethodType)
+                    or isinstance(val, staticmethod))
+        }
+
+        tap_class_dict_keys = Tap().__dict__.keys() | Tap.__dict__.keys()
+        stored_dict = {key: stored_dict[key] for key in stored_dict.keys() - tap_class_dict_keys}
+
+        return stored_dict
 
     def from_dict(self, args_dict: Dict[str, Any], skip_unsettable: bool = False) -> None:
         """Loads arguments from a dictionary, ensuring all required arguments are set.
@@ -418,11 +461,11 @@ class Tap(ArgumentParser):
         # All of the required arguments must be provided or already set
         required_args = {a.dest for a in self._actions if a.required}
         unprovided_required_args = required_args - args_dict.keys()
+        missing_required_args = [arg for arg in unprovided_required_args if not hasattr(self, arg)]
 
-        for arg in unprovided_required_args:
-            if not hasattr(self, arg):
-                raise ValueError(f'Input dictionary "{args_dict}" does not include '
-                                 f'all unset required argument "{arg}".')
+        if len(missing_required_args) > 0:
+            raise ValueError(f'Input dictionary "{args_dict}" does not include '
+                             f'all unset required arguments: "{missing_required_args}".')
 
         # Load all arguments
         for key, value in args_dict.items():
@@ -444,19 +487,24 @@ class Tap(ArgumentParser):
         with open(path, 'w') as f:
             json.dump(self._log_all(), f, indent=4, sort_keys=True, cls=PythonObjectEncoder)
 
-    def load(self, path: str, check_reproducibility: bool = False) -> None:
+    def load(self,
+             path: str,
+             check_reproducibility: bool = False,
+             skip_unsettable: bool = False) -> None:
         """Loads the arguments in JSON format. Note: Due to JSON, tuples are loaded as lists.
 
         :param path: Path to the JSON file where the arguments will be loaded from.
         :param check_reproducibility: When True, raises an error if the loaded reproducibility
                                       information doesn't match the current reproducibility information.
+        :param skip_unsettable: When True, skips attributes that cannot be set in the Tap object,
+                                e.g. properties without setters.
         """
         with open(path) as f:
             args_dict = json.load(f, object_hook=as_python_object)
 
         # Remove loaded reproducibility information since it is no longer valid
         reproducibility = args_dict.pop('reproducibility', None)
-        
+
         if check_reproducibility:
             no_reproducibility_message = 'Reproducibility not guaranteed'
             current_reproducibility = self.get_reproducibility_info()
@@ -485,7 +533,7 @@ class Tap(ArgumentParser):
                 raise ValueError(f'{no_reproducibility_message}: Uncommitted changes '
                                  f'in current args.')
 
-        self.from_dict(args_dict)
+        self.from_dict(args_dict, skip_unsettable=skip_unsettable)
 
     def __str__(self) -> str:
         """Returns a string representation of self.
